/*********************************************************************
* Liferrumgroup.c
*     Li-Fe  LiFePo4 
* : 2.03.2023 12:06:51
* :  .
**********************************************************************/

// 
#include <avr/io.h>
#include <util/delay_basic.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "Main.h"
#include "Liferrumgroup.h"
#include "Operatingmode.h"
#include "Systemcontrol.h"

bit_lfg indx_lfg={0,0}; //  
var_lfg alter_lfg={{0,0},{0,0},{0,0},{0,0},{0,0},{0,0,0,0},{0,0,0,0}}; //  

PROGMEM const byte Lfg_DischargeBattery[3][2][4]={{{5,0,64,0}, //   : I<=0,2 U>=2800  <40*
											{10,1,0,100}}, // : I=0,1C U>=2400   I<0.01C  10   
											{{3,0,64,0}, //   : I<=0,33 U>=2800  <40*
											{10,1,0,100}}, // : I=0,1C U>=2400   I<0.01C  10   
											{{2,0,64,0}, //   : I<=0,5 U>=2800  <40*
											{5,1,0,100}}}; // : I=0,2C U>=2400   I<0.01C  10   
PROGMEM const byte Lfg_ChargeBattery[3][2][4]={{{5,0,64,0}, //    1: I<=0,2C U<=3500  <40*C
											{10,1,0,100}}, // : I<=0,1C U<=3600   I<0,01C  10   
											{{3,0,64,0}, //    1: I<=0,33C U<=3500  <40*C
											{10,1,0,100}}, // : I<=0,1C U<=3600   I<0,01C  10    
											{{1,0,64,0}, //    1: I<=1C U<=3500  <40*C
											{5,1,0,100}}}; // : I<=0,2C U<=3600   I<0,01C  10   

void LFG_DefiningParameters(bit indxnumberbattery_prm,bit indxchargedischarge_prm,word settcapacitybattery_prm,byte currentmethodbattery_prm,word numberbatterysources_prm) //    Li-Fe,LiFePo4 
	{
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: indx_lfg.updatingfirstdata=TRUE; //       1
					alter_lfg.currentmethodbattery[indxnumberbattery_prm]=currentmethodbattery_prm; //    1
					if(alter_lfg.numbercellsbattery[indxnumberbattery_prm]==0) alter_lfg.numbercellsbattery[indxnumberbattery_prm]=numberbatterysources_prm; //    
					break;
		case TRUE: indx_lfg.updatingseconddata=TRUE; //       2
					alter_lfg.currentmethodbattery[indxnumberbattery_prm]=currentmethodbattery_prm; //    2
					if(alter_lfg.numbercellsbattery[indxnumberbattery_prm]==0) alter_lfg.numbercellsbattery[indxnumberbattery_prm]=numberbatterysources_prm; //    
		}
	switch(indxchargedischarge_prm) //  /
		{
		case FALSE: LFG_ChargeParameters(indxnumberbattery_prm,settcapacitybattery_prm); //   Li-Fe,LiFePo4 
					break;
		case TRUE: LFG_DischargeParameters(indxnumberbattery_prm,settcapacitybattery_prm); //   Li-Fe,LiFePo4 
		}
	}

word LFG_DetermCurrentcontrol(bit indxnumberbattery_prm,bit indxchargedischarge_prm,word measuredcurrentbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredtemperature_prm) //      
	{
	word settingcurrentbattery_tmp=0; // 
	switch(indxchargedischarge_prm) // /
		{
		case FALSE: settingcurrentbattery_tmp=LFG_ChargeController(indxnumberbattery_prm,measuredcurrentbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredtemperature_prm); //  
					break;
		case TRUE: settingcurrentbattery_tmp=LFG_DischargeController(indxnumberbattery_prm,measuredcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredtemperature_prm); //  
		}
	return settingcurrentbattery_tmp; //    /
	}

void LFG_ResetVariables(bit indxnumberbattery_prm) //  
	{
	byte counterindxarray_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: while(counterindxarray_tmp<4) //     1 
						{
						alter_lfg.limitfirstparameters[counterindxarray_tmp]=0; //  
						counterindxarray_tmp++; //   
						}
					if(indx_lfg.updatingfirstdata==TRUE) indx_lfg.updatingfirstdata=FALSE; //      
					break;
		case TRUE: while(counterindxarray_tmp<4) //     2 
			{
			alter_lfg.limitsecondparameters[counterindxarray_tmp]=0; //  
			counterindxarray_tmp++; //   
			}
		if(indx_lfg.updatingseconddata==TRUE) indx_lfg.updatingseconddata=FALSE; //      
		}
	alter_lfg.countergeneralusage[indxnumberbattery_prm]=0; //    
	alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=0; //   / 
	alter_lfg.currentstatusprocess[indxnumberbattery_prm]=0; //   
	alter_lfg.currentmethodbattery[indxnumberbattery_prm]=0; //    
	alter_lfg.numbercellsbattery[indxnumberbattery_prm]=0; //       
	}

void LFG_ChargeParameters(bit indxnumberbattery_prm,word settcapacitybattery_prm) //   
	{
	byte algoritmdata_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][0]); //     1
					alter_lfg.limitfirstparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    1
					if(alter_lfg.limitfirstparameters[0]>LFGMAXLIMIT_CURRENT) alter_lfg.limitfirstparameters[0]=LFGMAXLIMIT_CURRENT; //     2300  
					algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][1]); //   
					if(algoritmdata_tmp==0) alter_lfg.limitfirstparameters[1]=LFGMAXBATT_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //             
					else alter_lfg.limitfirstparameters[1]=LFGMAXFINAL_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //             
					algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][2]); //   
					alter_lfg.limitfirstparameters[2]=(word)algoritmdata_tmp*10; //     40* 
					algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //      
						{
						alter_lfg.limitfirstparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //   
						if(alter_lfg.limitfirstparameters[3]<LFGMINLIMIT_CURRENT) alter_lfg.limitfirstparameters[3]=LFGMINLIMIT_CURRENT; //      10  
						}
					else alter_lfg.limitfirstparameters[3]=algoritmdata_tmp; //    
					break;
		case TRUE: algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][0]); //     2
					alter_lfg.limitsecondparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    2
					if(alter_lfg.limitsecondparameters[0]>LFGMAXLIMIT_CURRENT) alter_lfg.limitsecondparameters[0]=LFGMAXLIMIT_CURRENT; //     2300  
					algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][1]); //    
					if(algoritmdata_tmp==0) alter_lfg.limitsecondparameters[1]=LFGMAXBATT_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //              
					else alter_lfg.limitsecondparameters[1]=LFGMAXFINAL_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //             
					algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][2]); //   
					alter_lfg.limitsecondparameters[2]=(word)algoritmdata_tmp*10; //     40* 
					algoritmdata_tmp=pgm_read_byte(&Lfg_ChargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lfg.limitsecondparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //   
						if(alter_lfg.limitsecondparameters[3]<LFGMINLIMIT_CURRENT) alter_lfg.limitsecondparameters[3]=LFGMINLIMIT_CURRENT; //      10   
						}
					else alter_lfg.limitsecondparameters[3]=algoritmdata_tmp; //   
		}
	}

void LFG_DischargeParameters(bit indxnumberbattery_prm,word settcapacitybattery_prm) //   
	{
	byte algoritmdata_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][0]); //     1 
					alter_lfg.limitfirstparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    1
					if(alter_lfg.limitfirstparameters[0]>LFGMAXLIMIT_CURRENT) alter_lfg.limitfirstparameters[0]=LFGMAXLIMIT_CURRENT; //     2300  
					algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][1]); //   
					if(algoritmdata_tmp==0) alter_lfg.limitfirstparameters[1]=LFGMINBATT_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //              
					else alter_lfg.limitfirstparameters[1]=LFGMINFINAL_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //            
					algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][2]); //    
					alter_lfg.limitfirstparameters[2]=(word)algoritmdata_tmp*10; //     40* 
					algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //      
						{
						alter_lfg.limitfirstparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lfg.limitfirstparameters[3]<LFGMINLIMIT_CURRENT) alter_lfg.limitfirstparameters[3]=LFGMINLIMIT_CURRENT; //      10  
						}
					else alter_lfg.limitfirstparameters[3]=algoritmdata_tmp; //  
					break;
		case TRUE: algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][0]); //   
					alter_lfg.limitsecondparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    2
					if(alter_lfg.limitsecondparameters[0]>LFGMAXLIMIT_CURRENT) alter_lfg.limitsecondparameters[0]=LFGMAXLIMIT_CURRENT; //     2300  
					algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][1]); //   
					if(algoritmdata_tmp==0) alter_lfg.limitsecondparameters[1]=LFGMINBATT_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //              
					else alter_lfg.limitsecondparameters[1]=LFGMINFINAL_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //             
					algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][2]); //   
					alter_lfg.limitsecondparameters[2]=(word)algoritmdata_tmp*10; //     40* 
					algoritmdata_tmp=pgm_read_byte(&Lfg_DischargeBattery[alter_lfg.currentmethodbattery[indxnumberbattery_prm]][alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lfg.limitsecondparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lfg.limitsecondparameters[3]<LFGMINLIMIT_CURRENT) alter_lfg.limitsecondparameters[3]=LFGMINLIMIT_CURRENT; //      10   
						}
					else alter_lfg.limitsecondparameters[3]=algoritmdata_tmp; //  
		}
	}

word LFG_ChargeController(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredtemperature_prm) //  
	{
	word settingcurrentbattery_tmp=0; // 
	word limitcurrentparameters_tmp=0;
	word limitvoltageparameters_tmp=0;
	word intresistancebattery_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: switch(alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]) //    1
						{
						case 0:	intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     
								switch(alter_lfg.currentmethodbattery[indxnumberbattery_prm]) //  
									{
									case 0: limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //       
											break;
									case 1: if((intresistancebattery_tmp<300)&&(measuredtemperature_prm<alter_lfg.limitfirstparameters[2])) limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //    1,5           0,3     40*
											else limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //           0,3      40*  
											break;
									case 2: if(measuredtemperature_prm>=alter_lfg.limitfirstparameters[2]) //   40*  
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]/3; //   3          0,3 
												else limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]/4; //   4      0,3     
												}
											else //    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //         0,3 
												else limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]/2; //   2      0,3     
												}
									}
								limitvoltageparameters_tmp=alter_lfg.limitfirstparameters[1]; //        
								settingcurrentbattery_tmp=LFG_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=2) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=2; // :   
								if(measuredvoltagebattery_prm>=alter_lfg.limitfirstparameters[1]) //         
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //      
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]++; //      
									indx_lfg.updatingfirstdata=FALSE; //     
									}
								else if((measuredchargebattery_prm>4)&&(measuredvoltagebattery_prm<LFGMINFINAL_VOLTAGE)) //   5     1    
									{
									settingcurrentbattery_tmp=0; //          
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lfg.currentstatusprocess[indxnumberbattery_prm]=9; // :   
									indx_lfg.updatingfirstdata=FALSE; //     
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //   
								limitvoltageparameters_tmp=alter_lfg.limitfirstparameters[1]; //    
								settingcurrentbattery_tmp=LFG_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=3) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=3; // : 
								if((measuredcurrentbattery_prm<=alter_lfg.limitfirstparameters[3])&&(measuredvoltagebattery_prm>=alter_lfg.limitfirstparameters[1])) //             
									{
									settingcurrentbattery_tmp=0; //          1
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lfg.currentstatusprocess[indxnumberbattery_prm]=4; // :    
									indx_lfg.updatingfirstdata=FALSE; //     
									}
						}
					break;
		case TRUE: switch(alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]) //    2
						{
						case 0: intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     
								switch(alter_lfg.currentmethodbattery[TRUE]) //  
									{
									case 0: limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //       
											break;
									case 1: if((intresistancebattery_tmp<300)&&(measuredtemperature_prm<alter_lfg.limitsecondparameters[2])) limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //    1,5           0,3     40*
											else limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //           0,3      40*  
											break;
									case 2: if(measuredtemperature_prm>=alter_lfg.limitsecondparameters[2]) //   40*  
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]/3; //   3          0,3 
												else limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]/4; //   4      0,3    
												}
											else //    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //         0,3 
												else limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]/2; //   2      0,3    
												}
									}
								limitvoltageparameters_tmp=alter_lfg.limitsecondparameters[1]; //         
								settingcurrentbattery_tmp=LFG_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=2) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=2; // :  
								if(measuredvoltagebattery_prm>=alter_lfg.limitsecondparameters[1]) //         
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //      
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]++; //      
									indx_lfg.updatingseconddata=FALSE; //     
									}
								else if((measuredchargebattery_prm>4)&&(measuredvoltagebattery_prm<LFGMINFINAL_VOLTAGE)) //   5       
									{
									settingcurrentbattery_tmp=0; //          2
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lfg.currentstatusprocess[indxnumberbattery_prm]=9; // :   
									indx_lfg.updatingseconddata=FALSE; //     
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //    
								limitvoltageparameters_tmp=alter_lfg.limitsecondparameters[1]; //    
								settingcurrentbattery_tmp=LFG_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=3) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=3; // : 
								if((measuredcurrentbattery_prm<=alter_lfg.limitsecondparameters[3])&&(measuredvoltagebattery_prm>=alter_lfg.limitsecondparameters[1])) //            
									{
									settingcurrentbattery_tmp=0; //          2
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lfg.currentstatusprocess[indxnumberbattery_prm]=4; // :   
									indx_lfg.updatingseconddata=FALSE; //     
									}
						}
		}
	return settingcurrentbattery_tmp; //     
	}

word LFG_DischargeController(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredtemperature_prm) //  
	{
	word settingcurrentbattery_tmp=0; // 
	word limitcurrentparameters_tmp=0;
	word limitvoltageparameters_tmp=0;
	word intresistancebattery_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: switch(alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]) //    1
						{
						case 0: intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     
								switch(alter_lfg.currentmethodbattery[indxnumberbattery_prm]) //  
									{
									case 0 ... 1: limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //         
											break;
									case 2: if(measuredtemperature_prm>=alter_lfg.limitfirstparameters[2]) limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]/2; //    2     40*     
											else //E    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //         0,3 
												else limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]*2/3; //    1,5      0,3   
												}
									}
								limitvoltageparameters_tmp=alter_lfg.limitfirstparameters[1]; //         
								settingcurrentbattery_tmp=LFG_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=6) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=6; // :  
								if(measuredvoltagebattery_prm<=alter_lfg.limitfirstparameters[1]) //          
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]++; //      
									indx_lfg.updatingfirstdata=FALSE; //     
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lfg.limitfirstparameters[0]; //    
								limitvoltageparameters_tmp=alter_lfg.limitfirstparameters[1]; //        
								settingcurrentbattery_tmp=LFG_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=7) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=7; // : 
								if((measuredcurrentbattery_prm<=alter_lfg.limitfirstparameters[3])&&(measuredvoltagebattery_prm<=alter_lfg.limitfirstparameters[1])) //             
									{
									settingcurrentbattery_tmp=0; //          1
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lfg.currentstatusprocess[indxnumberbattery_prm]=8; // :   
									indx_lfg.updatingfirstdata=FALSE; //     
									}
						}
					break;
		case TRUE: switch(alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]) //    2
						{
						case 0: intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     
								switch(alter_lfg.currentmethodbattery[indxnumberbattery_prm]) //  
									{
									case 0 ... 1: limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //         
											break;
									case 2: if(measuredtemperature_prm>=alter_lfg.limitsecondparameters[2]) limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]/2; //    2     40*     
											else //E    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //         0,3 
												else limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]*2/3; //    1,5      0,3   
												}
									}
								limitvoltageparameters_tmp=alter_lfg.limitsecondparameters[1]; //      2800   2
								settingcurrentbattery_tmp=LFG_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=6) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=6; // :  
								if(measuredvoltagebattery_prm<=alter_lfg.limitsecondparameters[1]) //   2         
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]++; //      
									indx_lfg.updatingseconddata=FALSE; //     
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lfg.limitsecondparameters[0]; //    
								limitvoltageparameters_tmp=alter_lfg.limitsecondparameters[1]; //       
								settingcurrentbattery_tmp=LFG_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lfg.currentstatusprocess[indxnumberbattery_prm]!=7) alter_lfg.currentstatusprocess[indxnumberbattery_prm]=7; // : 
								if((measuredcurrentbattery_prm<=alter_lfg.limitsecondparameters[3])&&(measuredvoltagebattery_prm<=alter_lfg.limitsecondparameters[1])) //             
									{
									settingcurrentbattery_tmp=0; //          2
									alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lfg.currentstatusprocess[indxnumberbattery_prm]=8; // :   
									indx_lfg.updatingseconddata=FALSE; //     
									}
						}
		}
	return settingcurrentbattery_tmp; //    
	}

word LFG_ChargingCurrent(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word settingcurrentbattery_prm,word limitcurrentparameters_prm,word limitvoltageparameters_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm) //   
	{
	byte stepincreasecurrent_tmp=0; // 
	word settingcurrentbattery_tmp=0; 
	word currentvaluepower_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	currentvaluepower_tmp=LFG_DefinitionPower(measuredcurrentbattery_prm,measuredvoltagebattery_prm); //     
	if((measuredcurrentbattery_prm<limitcurrentparameters_prm)&&(measuredvoltagebattery_prm<limitvoltageparameters_prm)&&(measvoltageloadedbattery_prm<(LFGMAXLOAD_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]))&&(currentvaluepower_tmp<LFGMAXPOWER_CHARGE)) //           
		{
		switch(alter_lfg.currentmethodbattery[indxnumberbattery_prm]) //  
			{
			case 0: stepincreasecurrent_tmp=1; //        1
					break;
			case 1: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/3))&&(alter_lfg.currentstatusprocess[indxnumberbattery_prm]==2)) stepincreasecurrent_tmp=3; //          33%       3
					else stepincreasecurrent_tmp=1; //   33%       1
					break;
			case 2: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/2))&&(alter_lfg.currentstatusprocess[indxnumberbattery_prm]==2)) stepincreasecurrent_tmp=5; //          50%       5
					else if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/3))&&(alter_lfg.currentstatusprocess[indxnumberbattery_prm]==2)) stepincreasecurrent_tmp=3; //          33%       3
					else stepincreasecurrent_tmp=1; //   33%       1
			}
		settingcurrentbattery_tmp+=stepincreasecurrent_tmp; //      
		}
	else if(((measuredcurrentbattery_prm>limitcurrentparameters_prm)||(measuredvoltagebattery_prm>limitvoltageparameters_prm)||(measvoltageloadedbattery_prm>=(LFGMAXLOAD_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]))||(currentvaluepower_tmp>LFGMAXPOWER_CHARGE))&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp--; //            
	else asm("nop"); //  
	return settingcurrentbattery_tmp; //    
	}

word LFG_DischargingCurrent(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word settingcurrentbattery_prm,word limitcurrentparameters_prm,word limitvoltageparameters_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm) //   
	{
	byte stepincreasecurrent_tmp=0; // 
	word settingcurrentbattery_tmp=0; 
	word currentvaluepower_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	currentvaluepower_tmp=LFG_DefinitionPower(measuredcurrentbattery_prm,measuredvoltagebattery_prm); //     
	if((measuredcurrentbattery_prm<limitcurrentparameters_prm)&&(measuredvoltagebattery_prm>limitvoltageparameters_prm)&&(measvoltageloadedbattery_prm>(LFGMINLOAD_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]))&&(currentvaluepower_tmp<LFGMAXPOWER_DISCHARGE)) //            
		{
		switch(alter_lfg.currentmethodbattery[indxnumberbattery_prm]) //  
			{
			case 0: stepincreasecurrent_tmp=1; //        1
					break;
			case 1: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/3))&&(alter_lfg.currentstatusprocess[indxnumberbattery_prm]==6)) stepincreasecurrent_tmp=2; //          33%       2
					else stepincreasecurrent_tmp=1; //   33%       1
					break;
			case 2: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/2))&&(alter_lfg.currentstatusprocess[indxnumberbattery_prm]==6)) stepincreasecurrent_tmp=3; //          50%       3
					else if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/3))&&(alter_lfg.currentstatusprocess[indxnumberbattery_prm]==6)) stepincreasecurrent_tmp=2; //          33%       2
					else stepincreasecurrent_tmp=1; //   33%       1
			}
		settingcurrentbattery_tmp+=stepincreasecurrent_tmp; //      
		}
	else //    
		{
		if((limitvoltageparameters_prm==(LFGMINFINAL_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]))&&(measuredvoltagebattery_prm<limitvoltageparameters_prm)&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp/=2; //        
		else if(((measuredcurrentbattery_prm>limitcurrentparameters_prm)||(measuredvoltagebattery_prm<limitvoltageparameters_prm)||(measvoltageloadedbattery_prm<=(LFGMINLOAD_VOLTAGE*alter_lfg.numbercellsbattery[indxnumberbattery_prm]))||(currentvaluepower_tmp>LFGMAXPOWER_DISCHARGE))&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp--; //          
		else asm("nop"); //  
		}
	return settingcurrentbattery_tmp; //   
	}

byte LFG_ReadcurrentStatus(bit indxnumberbattery_prm,bit indxparameters_prm) //         
	{
	byte currentparameters_tmp=0; //  
	switch(indxparameters_prm) //  
		{
		case FALSE: currentparameters_tmp=alter_lfg.currentstatusprocess[indxnumberbattery_prm]; //   
					break;
		case TRUE: currentparameters_tmp=alter_lfg.numbercellsbattery[indxnumberbattery_prm]; //     
		}
	return currentparameters_tmp; //   
	}

void LFG_WritedataAlgorithm(bit indxnumberbattery_prm,byte datadeferred_prm,byte statusdeferred_prm) //       
	{
	alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]=datadeferred_prm; //   
	alter_lfg.currentstatusprocess[indxnumberbattery_prm]=statusdeferred_prm; //  
	}

byte LFG_ReadcurrentAlgorithm(bit indxnumberbattery_prm) //         
	{
	return alter_lfg.countercyclesalgorithm[indxnumberbattery_prm]; //   
	}

bit LFG_ReadindxUpdatig(bit indxnumberbattery_prm) //     
	{
	bit indxupdatingdata_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: indxupdatingdata_tmp=indx_lfg.updatingfirstdata; //      1
					break;
		case TRUE: indxupdatingdata_tmp=indx_lfg.updatingseconddata; //      2
		}
	return indxupdatingdata_tmp; //  
	}

word LFG_DefinitionPower(word measuredcurrentbattery_prm,word measuredvoltagebattery_prm) //      
	{
	dword currentvaluepower_tmp=0; //  
	currentvaluepower_tmp=(dword)measuredcurrentbattery_prm*(dword)measuredvoltagebattery_prm/1000; //   
	return (word)currentvaluepower_tmp; //   
	}
















