/*******************************************************************************************
* Lithiumgroup.c
*     Li-Po,Li-Ion,Li-Mn,Li-Co,Li-Nca,Li-Nmc  
* : 08.12.2022 16:54:32
* :  .
********************************************************************************************/

// 
#include <avr/io.h>
#include <util/delay_basic.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "Main.h"
#include "Lithiumgroup.h"
#include "Operatingmode.h"
#include "Systemcontrol.h"

bit_lgr indx_lgr={0,0}; //  
var_lgr alter_lgr={{0,0},{0,0},{0,0},{0,0},{0,0},{0,0,0,0},{0,0,0,0}}; //  

PROGMEM const byte Lgr_DischargeBattery[3][2][4]={{{5,1,64,0}, //   : I<=0,2 U>3300  <40*
												{10,0,0,100}}, // : I=0,1C U>=3000   I<0.01C  10   
												{{3,1,64,0}, //   : I<=0,33 U>3300  <40*
												{10,0,0,100}}, //  : I=0,1C U>=3000   I<0.01C  10   
												{{2,1,64,0}, //   : I<=0,5 U>3300  <40*
												{5,0,0,100}}}; // : I=0,2C U>=3000   I<0.01C  10   
PROGMEM const byte Lgr_ChargeBattery[3][2][4]={{{5,1,64,0}, //   : I<=0,2C U<=4150  <40*C
												{10,0,0,100}}, //  : I<=0,1C U<=4200   I<0,01C  10   
												{{3,1,64,0}, //   : I<=0,33C U<=4150  <40*C
												{10,0,0,100}}, //  : I<=0,1C U<=4200   I<0,01C  10   
												{{1,1,64,0}, //   : I<=1C U<=4150  <40*C
												{5,0,0,100}}}; //  : I<=0,2C U<=4200   I<0,01C  10   

void LGR_DefiningParameters(bit indxnumberbattery_prm,bit indxchargedischarge_prm,word settcapacitybattery_prm,byte currentmethodbattery_prm,word numberbatterysources_prm) //    Li-Po,Li-Ion,Li-Mn,Li-Co,Li-Nca,Li-Nmc  
	{
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: indx_lgr.updatingfirstdata=TRUE; //       1
					alter_lgr.currentmethodbattery[indxnumberbattery_prm]=currentmethodbattery_prm; //    1
					if(alter_lgr.numbercellsbattery[indxnumberbattery_prm]==0) alter_lgr.numbercellsbattery[indxnumberbattery_prm]=numberbatterysources_prm; //    
					break;
		case TRUE: indx_lgr.updatingseconddata=TRUE; //       2
					alter_lgr.currentmethodbattery[indxnumberbattery_prm]=currentmethodbattery_prm; //    2
					if(alter_lgr.numbercellsbattery[indxnumberbattery_prm]==0) alter_lgr.numbercellsbattery[indxnumberbattery_prm]=numberbatterysources_prm; //    
		}
	switch(indxchargedischarge_prm) //  / 
		{
		case FALSE: LGR_ChargeParameters(indxnumberbattery_prm,settcapacitybattery_prm); //   Li-Po,Li-Ion,Li-Mn,Li-Co,Li-Nca,Li-Nmc  
					break; 
		case TRUE: LGR_DischargeParameters(indxnumberbattery_prm,settcapacitybattery_prm); //   Li-Po,Li-Ion,Li-Mn,Li-Co,Li-Nca,Li-Nmc  
		}
	}

word LGR_DetermCurrentcontrol(bit indxnumberbattery_prm,bit indxchargedischarge_prm,word measuredcurrentbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredtemperature_prm) //     
	{
	word settingcurrentbattery_tmp=0; // 
	switch(indxchargedischarge_prm) //  /
		{
		case FALSE: settingcurrentbattery_tmp=LGR_ChargeController(indxnumberbattery_prm,measuredcurrentbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredtemperature_prm); //  
					break;
		case TRUE: settingcurrentbattery_tmp=LGR_DischargeController(indxnumberbattery_prm,measuredcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredtemperature_prm); //  
		}
	return settingcurrentbattery_tmp; //    /
	}

void LGR_ResetVariables(bit indxnumberbattery_prm) //  
	{
	byte counterindxarray_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: while(counterindxarray_tmp<4) //     1
						{
						alter_lgr.limitfirstparameters[counterindxarray_tmp]=0; //  
						counterindxarray_tmp++; //   
						}
					if(indx_lgr.updatingfirstdata==TRUE) indx_lgr.updatingfirstdata=FALSE; //     
					break;
		case TRUE: while(counterindxarray_tmp<4) //     2
						{
						alter_lgr.limitsecondparameters[counterindxarray_tmp]=0; //  
						counterindxarray_tmp++; //   
						}
					if(indx_lgr.updatingseconddata==TRUE) indx_lgr.updatingseconddata=FALSE; //     
		}
	alter_lgr.countergeneralusage[indxnumberbattery_prm]=0; //   
	alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=0; //   / 
	alter_lgr.currentstatusprocess[indxnumberbattery_prm]=0; //  
	alter_lgr.currentmethodbattery[indxnumberbattery_prm]=0; //    
	alter_lgr.numbercellsbattery[indxnumberbattery_prm]=0; //       
	}

void LGR_DischargeParameters(bit indxnumberbattery_prm,word settcapacitybattery_prm) //   
	{
	byte algoritmdata_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][0]); //   
					alter_lgr.limitfirstparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    1
					if(alter_lgr.limitfirstparameters[0]>LGRMAXLIMIT_CURRENT) alter_lgr.limitfirstparameters[0]=LGRMAXLIMIT_CURRENT; //     2300 
					algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][1]); //   
					if(algoritmdata_tmp>0) alter_lgr.limitfirstparameters[1]=LGRMINBATT_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //     1       
					else alter_lgr.limitfirstparameters[1]=LGRMINFINAL_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //       
					algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][2]); //   
					alter_lgr.limitfirstparameters[2]=(word)algoritmdata_tmp*10; //    40*  1
					algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lgr.limitfirstparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lgr.limitfirstparameters[3]<LGRMINLIMIT_CURRENT) alter_lgr.limitfirstparameters[3]=LGRMINLIMIT_CURRENT; //      10  
						}
					else alter_lgr.limitfirstparameters[3]=algoritmdata_tmp; //  
					break;
		case TRUE: algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][0]); //   
					alter_lgr.limitsecondparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    2
					if(alter_lgr.limitsecondparameters[0]>LGRMAXLIMIT_CURRENT) alter_lgr.limitsecondparameters[0]=LGRMAXLIMIT_CURRENT; //     2300 
					algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][1]); //   
					if(algoritmdata_tmp>0) alter_lgr.limitsecondparameters[1]=LGRMINBATT_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //     2     
					else alter_lgr.limitsecondparameters[1]=LGRMINFINAL_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //       
					algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][2]); //   
					alter_lgr.limitsecondparameters[2]=(word)algoritmdata_tmp*10; //    40*  2
					algoritmdata_tmp=pgm_read_byte(&Lgr_DischargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lgr.limitsecondparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lgr.limitsecondparameters[3]<LGRMINLIMIT_CURRENT) alter_lgr.limitsecondparameters[3]=LGRMINLIMIT_CURRENT; //      10  
						}
					else alter_lgr.limitsecondparameters[3]=algoritmdata_tmp; //  
		}
	}

void LGR_ChargeParameters(bit indxnumberbattery_prm,word settcapacitybattery_prm) //   
	{
	byte algoritmdata_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][0]); //   
					alter_lgr.limitfirstparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    1
					if(alter_lgr.limitfirstparameters[0]>LGRMAXLIMIT_CURRENT) alter_lgr.limitfirstparameters[0]=LGRMAXLIMIT_CURRENT; //    1  2300 
					algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][1]); //   
					if(algoritmdata_tmp>0) alter_lgr.limitfirstparameters[1]=LGRMAXBATT_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //     1         
					else alter_lgr.limitfirstparameters[1]=LGRMAXFINAL_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //      
					algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][2]); //   
					alter_lgr.limitfirstparameters[2]=(word)algoritmdata_tmp*10; //    40*  1
					algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lgr.limitfirstparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lgr.limitfirstparameters[3]<LGRMINLIMIT_CURRENT) alter_lgr.limitfirstparameters[3]=LGRMINLIMIT_CURRENT; //      10  
						}
					else alter_lgr.limitfirstparameters[3]=algoritmdata_tmp; //  
					break;
		case TRUE: algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][0]); //   
					alter_lgr.limitsecondparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    2
					if(alter_lgr.limitsecondparameters[0]>LGRMAXLIMIT_CURRENT) alter_lgr.limitsecondparameters[0]=LGRMAXLIMIT_CURRENT; //    2  2300 
					algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][1]); //   
					if(algoritmdata_tmp>0) alter_lgr.limitsecondparameters[1]=LGRMAXBATT_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //     2         
					else alter_lgr.limitsecondparameters[1]=LGRMAXFINAL_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //      
					algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][2]); //   
					alter_lgr.limitsecondparameters[2]=(word)algoritmdata_tmp*10; //    40*  2
					algoritmdata_tmp=pgm_read_byte(&Lgr_ChargeBattery[alter_lgr.currentmethodbattery[indxnumberbattery_prm]][alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lgr.limitsecondparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lgr.limitsecondparameters[3]<LGRMINLIMIT_CURRENT) alter_lgr.limitsecondparameters[3]=LGRMINLIMIT_CURRENT; //      10  
						}
					else alter_lgr.limitsecondparameters[3]=algoritmdata_tmp; //  
		}
	}

word LGR_ChargeController(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredtemperature_prm) //  
	{
	word settingcurrentbattery_tmp=0; // 
	word limitcurrentparameters_tmp=0;
	word limitvoltageparameters_tmp=0;
	word intresistancebattery_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: switch(alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]) //    1
						{
						case 0:	intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     
								switch(alter_lgr.currentmethodbattery[indxnumberbattery_prm]) //  
									{
									case 0: limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]; //       
											break;
									case 1: if((intresistancebattery_tmp<300)&&(measuredtemperature_prm<alter_lgr.limitfirstparameters[2])) limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]*3/2; //    1,5           0,3     40*
											else limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]; //           0,3      40*  
											break;
									case 2: if(measuredtemperature_prm>=alter_lgr.limitfirstparameters[2]) //   40*  
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]/3; //   3          0,3  
												else limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]/4; //   4      0,3   
												}
											else //    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]; //        0,3  
												else limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]/2; //   2      0,3   
												}
									}
								limitvoltageparameters_tmp=alter_lgr.limitfirstparameters[1]; //        
								settingcurrentbattery_tmp=LGR_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=2) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=2; // :  
								if(measuredvoltagebattery_prm>=alter_lgr.limitfirstparameters[1]) //          
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]++; //       
									indx_lgr.updatingfirstdata=FALSE; //    
									}
								else if((measuredchargebattery_prm>4)&&(measuredvoltagebattery_prm<LGRMINFINAL_VOLTAGE)) //   5       
									{
									settingcurrentbattery_tmp=0; //        
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lgr.currentstatusprocess[indxnumberbattery_prm]=9; // :  
									indx_lgr.updatingfirstdata=FALSE; //    
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]; //   
								limitvoltageparameters_tmp=alter_lgr.limitfirstparameters[1]; //      
								settingcurrentbattery_tmp=LGR_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=3) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=3; // : 
								if((measuredcurrentbattery_prm<=alter_lgr.limitfirstparameters[3])&&(measuredvoltagebattery_prm>=alter_lgr.limitfirstparameters[1])) //            
									{
									settingcurrentbattery_tmp=0; //        
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lgr.currentstatusprocess[indxnumberbattery_prm]=4; // :   
									indx_lgr.updatingfirstdata=FALSE; //    
									}
						}
					break;
		case TRUE: switch(alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]) //    2
						{
						case 0: intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     2 
								switch(alter_lgr.currentmethodbattery[indxnumberbattery_prm]) //  
									{
									case 0: limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]; //       
											break;
									case 1: if((intresistancebattery_tmp<300)&&(measuredtemperature_prm<alter_lgr.limitsecondparameters[2])) limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]*3/2; //    1,5           0,3     40*
											else limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]; //           0,3      40*  
											break;
									case 2: if(measuredtemperature_prm>=alter_lgr.limitsecondparameters[2]) //   40*  
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]/3; //   3          0,3 
												else limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]/4; //   4      0,3   
												}
											else //    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]; //        0,3 
												else limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]/2; //   2      0,3   
												}
									}
								limitvoltageparameters_tmp=alter_lgr.limitsecondparameters[1]; //        
								settingcurrentbattery_tmp=LGR_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=2) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=2; // :  
								if(measuredvoltagebattery_prm>=alter_lgr.limitsecondparameters[1]) //         
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]++; //      
									indx_lgr.updatingseconddata=FALSE; //    
									}
								else if((measuredchargebattery_prm>4)&&(measuredvoltagebattery_prm<LGRMINFINAL_VOLTAGE)) //   5      
									{
									settingcurrentbattery_tmp=0; //        
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lgr.currentstatusprocess[indxnumberbattery_prm]=9; // :  
									indx_lgr.updatingseconddata=FALSE; //    
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]; //    0,25
								limitvoltageparameters_tmp=alter_lgr.limitsecondparameters[1]; //     
								settingcurrentbattery_tmp=LGR_ChargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=3) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=3; // : 
								if((measuredcurrentbattery_prm<=alter_lgr.limitsecondparameters[3])&&(measuredvoltagebattery_prm>=alter_lgr.limitsecondparameters[1])) //            
									{
									settingcurrentbattery_tmp=0; //        
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lgr.currentstatusprocess[indxnumberbattery_prm]=4; // :   
									indx_lgr.updatingseconddata=FALSE; //    
									}
						}
		}
	return settingcurrentbattery_tmp; //    
	}

word LGR_DischargeController(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredtemperature_prm) //  
	{
	word settingcurrentbattery_tmp=0; // 
	word limitcurrentparameters_tmp=0;
	word limitvoltageparameters_tmp=0;
	word intresistancebattery_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: switch(alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]) //    1
						{
						case 0: intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     
								switch(alter_lgr.currentmethodbattery[indxnumberbattery_prm]) //  
									{
									case 0 ... 1: limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]; //         
											break;
									case 2: if(measuredtemperature_prm>=alter_lgr.limitfirstparameters[2]) limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]/2; //    2     40*     
											else //E    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]; //         0,3 
												else limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]*2/3; //    1,5      0,3   
												}
									}
								limitvoltageparameters_tmp=alter_lgr.limitfirstparameters[1]; //          
								settingcurrentbattery_tmp=LGR_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=6) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=6; // :  
								if(measuredvoltagebattery_prm<=alter_lgr.limitfirstparameters[1]) //      
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]++; //      
									indx_lgr.updatingfirstdata=FALSE; //    
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lgr.limitfirstparameters[0]; //   
								limitvoltageparameters_tmp=alter_lgr.limitfirstparameters[1]; //     
								settingcurrentbattery_tmp=LGR_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=7) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=7; // : 
								if((measuredcurrentbattery_prm<=alter_lgr.limitfirstparameters[3])&&(measuredvoltagebattery_prm<=alter_lgr.limitfirstparameters[1])) //            
									{
									settingcurrentbattery_tmp=0; //        
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lgr.currentstatusprocess[indxnumberbattery_prm]=8; // :   
									indx_lgr.updatingfirstdata=FALSE; //    
									}
						}
					break;
		case TRUE: switch(alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]) //    2
						{
						case 0: intresistancebattery_tmp=OPR_ReadintResistance(indxnumberbattery_prm); //     
								switch(alter_lgr.currentmethodbattery[indxnumberbattery_prm]) //  
									{
									case 0 ... 1: limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]; //         
											break;
									case 2: if(measuredtemperature_prm>=alter_lgr.limitsecondparameters[2]) limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]/2; //    2     40*     
											else //E    40*
												{
												if(intresistancebattery_tmp<300) limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]; //         0,3 
												else limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]*2/3; //    1,5      0,3   
												}
									}
								limitvoltageparameters_tmp=alter_lgr.limitsecondparameters[1]; //          
								settingcurrentbattery_tmp=LGR_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=6) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=6; // :  
								if(measuredvoltagebattery_prm<=alter_lgr.limitsecondparameters[1]) //      
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]++; //      
									indx_lgr.updatingseconddata=FALSE; //    
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lgr.limitsecondparameters[0]; //    0,25
								limitvoltageparameters_tmp=alter_lgr.limitsecondparameters[1]; //      
								settingcurrentbattery_tmp=LGR_DischargingCurrent(indxnumberbattery_prm,measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lgr.currentstatusprocess[indxnumberbattery_prm]!=7) alter_lgr.currentstatusprocess[indxnumberbattery_prm]=7; // : 
								if((measuredcurrentbattery_prm<=alter_lgr.limitsecondparameters[3])&&(measuredvoltagebattery_prm<=alter_lgr.limitsecondparameters[1])) //            
									{
									settingcurrentbattery_tmp=0; //        
									alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=0; //   
									alter_lgr.currentstatusprocess[indxnumberbattery_prm]=8; // :   
									indx_lgr.updatingseconddata=FALSE; //    
									}
						}
		}
	return settingcurrentbattery_tmp; //    
	}

word LGR_ChargingCurrent(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word settingcurrentbattery_prm,word limitcurrentparameters_prm,word limitvoltageparameters_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm) //   
	{
	byte stepincreasecurrent_tmp=0; // 
	word settingcurrentbattery_tmp=0;
	word currentvaluepower_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	currentvaluepower_tmp=LGR_DefinitionPower(measuredcurrentbattery_prm,measuredvoltagebattery_prm); //     
	if((measuredcurrentbattery_prm<limitcurrentparameters_prm)&&(measuredvoltagebattery_prm<limitvoltageparameters_prm)&&(measvoltageloadedbattery_prm<(LGRMAXLOAD_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]))&&(currentvaluepower_tmp<LGRMAXPOWER_CHARGE)) //         
		{
		switch(alter_lgr.currentmethodbattery[indxnumberbattery_prm]) //  
			{
			case 0: stepincreasecurrent_tmp=1; //        1
					break;
			case 1: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/2))&&(alter_lgr.currentstatusprocess[indxnumberbattery_prm]==2)) stepincreasecurrent_tmp=2; //          50%      2
					else stepincreasecurrent_tmp=1; //   50%      1
					break;
			case 2: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/2))&&(alter_lgr.currentstatusprocess[indxnumberbattery_prm]==2)) stepincreasecurrent_tmp=3; //          50%      3
					else if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/3))&&(alter_lgr.currentstatusprocess[indxnumberbattery_prm]==2)) stepincreasecurrent_tmp=2; //          33%      2
					else stepincreasecurrent_tmp=1; //   33%      1
			}
		settingcurrentbattery_tmp+=stepincreasecurrent_tmp; //      
		}
	else if(((measuredcurrentbattery_prm>limitcurrentparameters_prm)||(measuredvoltagebattery_prm>limitvoltageparameters_prm)||(measvoltageloadedbattery_prm>=(LGRMAXLOAD_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]))||(currentvaluepower_tmp>LGRMAXPOWER_CHARGE))&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp--; //           
	else asm("nop"); //  
	return settingcurrentbattery_tmp; //   
	}

word LGR_DischargingCurrent(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word settingcurrentbattery_prm,word limitcurrentparameters_prm,word limitvoltageparameters_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm) //   
	{
	byte stepincreasecurrent_tmp=0; // 
	word settingcurrentbattery_tmp=0; 
	word currentvaluepower_tmp=0;
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	currentvaluepower_tmp=LGR_DefinitionPower(measuredcurrentbattery_prm,measuredvoltagebattery_prm); //     
	if((measuredcurrentbattery_prm<limitcurrentparameters_prm)&&(measuredvoltagebattery_prm>limitvoltageparameters_prm)&&(measvoltageloadedbattery_prm>(LGRMINLOAD_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]))&&(currentvaluepower_tmp<LGRMAXPOWER_DISCHARGE)) //          
		{
		switch(alter_lgr.currentmethodbattery[indxnumberbattery_prm]) //  
			{
			case 0: stepincreasecurrent_tmp=1; //        1
					break;
			case 1: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/3))&&(alter_lgr.currentstatusprocess[indxnumberbattery_prm]==6)) stepincreasecurrent_tmp=2; //          33%       2
					else stepincreasecurrent_tmp=1; //   33%       1
					break;
			case 2: if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/2))&&(alter_lgr.currentstatusprocess[indxnumberbattery_prm]==6)) stepincreasecurrent_tmp=3; //          50%       3
					else if((measuredcurrentbattery_prm<(limitcurrentparameters_prm-limitcurrentparameters_prm/3))&&(alter_lgr.currentstatusprocess[indxnumberbattery_prm]==6)) stepincreasecurrent_tmp=2; //          33%       2
					else stepincreasecurrent_tmp=1; //   33%       1
			}
		settingcurrentbattery_tmp+=stepincreasecurrent_tmp; //     
		}
	else //   
		{
		if((limitvoltageparameters_prm==(LGRMINFINAL_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]))&&(measuredvoltagebattery_prm<limitvoltageparameters_prm)&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp/=2; //        
		else if(((measuredcurrentbattery_prm>limitcurrentparameters_prm)||(measuredvoltagebattery_prm<limitvoltageparameters_prm)||(measvoltageloadedbattery_prm<=(LGRMINLOAD_VOLTAGE*alter_lgr.numbercellsbattery[indxnumberbattery_prm]))||(currentvaluepower_tmp>LGRMAXPOWER_DISCHARGE))&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp--; //            
		else asm("nop"); //  
		}
	return settingcurrentbattery_tmp; //   
	}

byte LGR_ReadcurrentStatus(bit indxnumberbattery_prm,bit indxparameters_prm) //   
	{
	byte dataparameters_tmp=0; // 
	switch(indxparameters_prm) //   
		{
		case FALSE: dataparameters_tmp=alter_lgr.currentstatusprocess[indxnumberbattery_prm]; //   
					break;
		case TRUE: dataparameters_tmp=(byte)alter_lgr.numbercellsbattery[indxnumberbattery_prm]; //     
		}
	return dataparameters_tmp; //  
	}

void LGR_WritedataAlgorithm(bit indxnumberbattery_prm,byte datadeferred_prm,byte statusdeferred_prm) //      
	{
	alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]=datadeferred_prm; //     
	alter_lgr.currentstatusprocess[indxnumberbattery_prm]=statusdeferred_prm; //  
	}

byte LGR_ReadcurrentAlgorithm(bit indxnumberbattery_prm) //      
	{
	return alter_lgr.countercyclesalgorithm[indxnumberbattery_prm]; //     
	}

bit LGR_ReadindxUpdatig(bit indxnumberbattery_prm) //      
	{
	bit indxupdatingdata_tmp=0; // 
	switch(indxnumberbattery_prm) //   
		{
		case FALSE: indxupdatingdata_tmp=indx_lgr.updatingfirstdata; //      1 
					break;
		case TRUE: indxupdatingdata_tmp=indx_lgr.updatingseconddata; //      2 
		}
	return indxupdatingdata_tmp; //  
	}

word LGR_DefinitionPower(word measuredcurrentbattery_prm,word measuredvoltagebattery_prm) //     
	{
	dword currentvaluepower_tmp=0; // 
	currentvaluepower_tmp=(dword)measuredcurrentbattery_prm*(dword)measuredvoltagebattery_prm/1000; //  
	return (word)currentvaluepower_tmp; //  
	}


















